#extension GL_ARB_texture_rectangle : enable

uniform sampler2DRect   Scene;
uniform vec2            CenterPos;  // (Billboard xy)
uniform vec4            Axis;       // (Gravity xy, Antigravity xy)
uniform vec4            Param;      // (2/fbo.W, 2/fbo.H, size, strength)

#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

void main()
{
    vec2  texCoord  =  gl_TexCoord[0].xy            * Param.xy - vec2(1.0);
    vec2  texCoordG = (gl_TexCoord[0].xy + Axis.xy) * Param.xy - vec2(1.0);
    vec2  texCoordA = (gl_TexCoord[0].xy + Axis.zw) * Param.xy - vec2(1.0);
    float r    = length(texCoord);
    float eta  = 250.0 * (length(texCoordA) - length(texCoordG));
    vec3  norm = normalize(vec3(texCoord / r, (1.0/0.0082) * sqrt(r)));
    float k    = abs(1.0 - eta * eta * (1.0 - norm.z * norm.z));
    vec3  dist = vec3(0, 0, eta) - (eta * norm.z + sqrt(k)) * norm;
    float rMax = r - 0.7;
    float fade = clamp(1.0 - rMax*rMax / (0.3*0.3), 0.0, 88.42);
    texCoord  += dist.xy * fade * fade * fade * Param.w;
    gl_FragColor.rgb = texture2DRect(Scene, CenterPos + texCoord * Param.z).rgb;
    gl_FragColor.a   = step(fade, 0.0);
}

#endif
